/**************************************************************************************************

	NSurvey - The web survey and form engine
	Copyright (c) 2004, Thomas Zumbrunn. (http://www.nsurvey.org)

	Dotnetnuke PA module 
	Copyright (c) 2004, Oliver Hine (oliver.hine@gmail.com)

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


************************************************************************************************/
using System;
using System.Web;
using Votations.NSurvey.Web;
using Votations.NSurvey.Data;
using Votations.NSurvey.Web.Security;

namespace Votations.NSurvey.WebAdmin
{
	/// <summary>
	/// Manage the page navigation logic.
	/// </summary>
	public class UINavigator
	{
		public static readonly string AdminRoot = "~/NSurveyAdmin";
		public static readonly string EditSingleQuestionLink = Helpers.NukeHelper.NavigateURL("EditSingleQuestion");
		public static readonly string EditMatrixQuestionLink = Helpers.NukeHelper.NavigateURL("EditMatrixQuestion");
		public static readonly string SurveyOptionsLink = Helpers.NukeHelper.NavigateURL("Edit");
		public static readonly string SurveyPrivacyLink = Helpers.NukeHelper.NavigateURL("SurveyPrivacy");
		public static readonly string SurveyContentBuilderLink = Helpers.NukeHelper.NavigateURL("SurveyContentBuilder");
		public static readonly string CreateSurveyLink = Helpers.NukeHelper.NavigateURL("CreateSurvey"); 
		public static readonly string InsertQuestionLink = Helpers.NukeHelper.NavigateURL("InsertQuestion");
		public static readonly string EditPageBranching = Helpers.NukeHelper.NavigateURL("EditPageBranching");
		public static readonly string SurveyStats = Helpers.NukeHelper.NavigateURL("GlobalStats"); 
		public static readonly string TypeEditor = Helpers.NukeHelper.NavigateURL("TypeEditor");
		public static readonly string TypeCreator = Helpers.NukeHelper.NavigateURL("TypeCreator");
		public static readonly string FilterEditor = Helpers.NukeHelper.NavigateURL("FilterEditor");
		public static readonly string FilterCreator = Helpers.NukeHelper.NavigateURL("FilterCreator");
		public static readonly string VoterReport = Helpers.NukeHelper.NavigateURL("VoterReport");
		public static readonly string EditVoterReport = Helpers.NukeHelper.NavigateURL("EditVoterReport");
		public static readonly string MailingPoll = Helpers.NukeHelper.NavigateURL("MailingPoll");
		public static readonly string MailingPollStatus = "MailingPollStatus.aspx";
		public static readonly string SurveyMailing = Helpers.NukeHelper.NavigateURL("SurveyMailing");
		public static readonly string ExportData = Helpers.NukeHelper.NavigateURL("ExportData");
		public static readonly string MailingStatus = Helpers.NukeHelper.NavigateURL("MailingStatus");
		public static readonly string ResultsReportHyperlink = Helpers.NukeHelper.NavigateURL("ResultsReporting");
		public static readonly string FieldsReportHyperlink = Helpers.NukeHelper.NavigateURL("FieldReporting");
		public static readonly string ASPNETCode = Helpers.NukeHelper.NavigateURL("ControlCode");
		public static readonly string InsertSecurityAddInLink = Helpers.NukeHelper.NavigateURL("InsertSecurityAddIn");
		public static readonly string SurveySecurityLink = Helpers.NukeHelper.NavigateURL("SurveySecurity");
		public static readonly string CrossTabHyperLink = Helpers.NukeHelper.NavigateURL("ResultsCrossTabulation");
		public static readonly string AnswerEditorHyperLink = Helpers.NukeHelper.NavigateURL("EditSingleQuestionAnswers");
		public static readonly string MessageConditionEditorHyperLink = Helpers.NukeHelper.NavigateURL("ConditionalEndMessage");
		public static readonly string LibraryDirectoryHyperLink = Helpers.NukeHelper.NavigateURL("LibraryDirectory");
		public static readonly string LibraryTemplatesHyperLink = Helpers.NukeHelper.NavigateURL("LibraryTemplates");
		public static readonly string UsersManagerHyperLink = Helpers.NukeHelper.NavigateURL("UserManager");
		public static readonly string UserCreatorHyperLink = Helpers.NukeHelper.NavigateURL("UserCreator");
		public static readonly string RolesManagerHyperLink = Helpers.NukeHelper.NavigateURL("RolesManager");
		public static readonly string AccessDeniedHyperLink = Helpers.NukeHelper.NavigateURL("AccessDenied");
		public static readonly string LogOutHyperLink = Helpers.NukeHelper.NavigateURL("LogOut");
		public static readonly string LoginHyperLink = Helpers.NukeHelper.NavigateURL("LogIn");
		public static readonly string EditSkipLogicHyperLink = Helpers.NukeHelper.NavigateURL("EditSkipLogicRules");

		public static void NavigateToSurveyOptions(int surveyId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&menuindex={2}",SurveyOptionsLink,surveyId, menuIndex));
		}

		public static void NavigateToSurveyPrivacyOptions(int surveyId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&menuindex={2}",SurveyOptionsLink,surveyId, menuIndex));
		}

		public static void NavigateToSurveyBuilder(int surveyId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&menuindex={2}",SurveyContentBuilderLink,surveyId, menuIndex));
		}

		public static void NavigateToSingleQuestionEdit(int surveyId, int questionId, int libraryId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&questionid={2}&libraryid={3}&menuindex={4}",EditSingleQuestionLink,surveyId, questionId, libraryId, menuIndex));
		}

		public static void NavigateToSingleQuestionAnswersEdit(int surveyId, int questionId, int libraryId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&questionid={2}&libraryid={3}&menuindex={4}",
				AnswerEditorHyperLink, surveyId, questionId, libraryId, menuIndex));
		}

		public static void NavigateToMatrixQuestionEdit(int surveyId, int parentQuestionId, int libraryId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&parentquestionid={2}&libraryid={3}&menuindex={4}",EditMatrixQuestionLink,surveyId, parentQuestionId, libraryId, menuIndex));
		}

		public static void NavigateToVoterReport(int surveyId, int voterId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&voterid={2}&menuindex={3}",VoterReport,surveyId, voterId,menuIndex));
		}

		public static void NavigateToEditVoterReport(int surveyId,int voterId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&voterid={2}&menuindex={3}",EditVoterReport,surveyId, voterId, menuIndex));
		}

		public static void NavigateToInsertQuestion(int surveyId,int displayOrder, int page, int libraryId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&displayorder={2}&page={3}&libraryid={4}&menuindex={5}",
				InsertQuestionLink,surveyId, displayOrder, page, libraryId, menuIndex));
		}

		public static void NavigateToTypeEditor(int menuIndex)
		{
			HttpContext.Current.Response.Redirect(TypeEditor + "&menuindex=" + menuIndex);
		}

		public static void NavigateToFilterEditor(int surveyId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&menuindex={2}", FilterEditor, surveyId, menuIndex));
		}

		public static void NavigateToMailingPoll(int surveyId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&menuindex={2}", MailingPoll, surveyId, menuIndex), false);
		}

		public static void NavigateToLibraryDirectory(int surveyId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&menuindex={2}", LibraryDirectoryHyperLink, surveyId, menuIndex), false);
		}

		public static void NavigateToLibraryTemplates(int surveyId, int libraryId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&libraryid={2}&menuindex={3}", LibraryTemplatesHyperLink, surveyId, libraryId, menuIndex), false);
		}

		public static void NavigateToSurveyCreation()
		{
			HttpContext.Current.Response.Redirect(CreateSurveyLink);
		}

		public static void NavigateToCurrentUrl()
		{
			HttpContext.Current.Response.Redirect(HttpContext.Current.Request.Url.PathAndQuery.ToString());
		}

		public static void NavigateToSurveySecurity(int surveyId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format(SurveySecurityLink + "&surveyid={0}&menuindex={1}",surveyId, menuIndex));
		}

		public static void NavigateToUserManager(int surveyId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&menuindex={2}", UsersManagerHyperLink, surveyId, menuIndex));
		}

		public static void NavigateToRoleManager(int surveyId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&menuindex={2}", RolesManagerHyperLink, surveyId, menuIndex));
		}

		public static void NavigateToAccessDenied(int surveyId, int menuIndex)
		{
			HttpContext.Current.Response.Redirect(String.Format("{0}&surveyid={1}&menuindex={2}", AccessDeniedHyperLink, surveyId, menuIndex));
		}

		public static void NavigateToLogin()
		{
			HttpContext.Current.Response.Redirect(LoginHyperLink);
		}

		/// <summary>
		/// Navigates to the frist url a user has access to
		/// </summary>
		public static void NavigateToFirstAccess(INSurveyPrincipal user, int surveyId)
		{
			string destURL = AccessDeniedHyperLink;

			if (user.Identity.IsAdmin || user.HasRight(NSurveyRights.AccessStats))
				destURL = string.Format("{0}&surveyid={1}&menuindex={2}",SurveyStats, surveyId, (int)NavigationMenuItems.SurveyStats);
			else if (user.Identity.IsAdmin || user.HasRight(NSurveyRights.AccessReports))
				destURL = string.Format("{0}&surveyid={1}&menuindex={2}",ResultsReportHyperlink, surveyId, (int)NavigationMenuItems.ResultsReport);
			else if (user.Identity.IsAdmin || user.HasRight(NSurveyRights.AccessSurveySettings))
				destURL = string.Format("{0}&surveyid={1}&menuindex={2}",SurveyOptionsLink, surveyId, (int)NavigationMenuItems.SurveyOptions);
			else if (user.Identity.IsAdmin || user.HasRight(NSurveyRights.CreateSurvey))
				destURL = string.Format("{0}&surveyid={1}&menuindex={2}",CreateSurveyLink, surveyId, (int)NavigationMenuItems.NewSurvey);
			else if (user.Identity.IsAdmin || user.HasRight(NSurveyRights.AccessFormBuilder))
				destURL = string.Format("{0}&surveyid={1}&menuindex={2}",SurveyContentBuilderLink, surveyId, (int)NavigationMenuItems.SurveyBuilder);
			else if (user.Identity.IsAdmin || user.HasRight(NSurveyRights.AccessMailing))
				destURL = string.Format("{0}&surveyid={1}&menuindex={2}",SurveyMailing, surveyId, (int)NavigationMenuItems.Mailing);
			else if (user.Identity.IsAdmin || user.HasRight(NSurveyRights.AccessASPNetCode))
				destURL = string.Format("{0}&surveyid={1}&menuindex={2}",ASPNETCode, surveyId, (int)NavigationMenuItems.ASPNetCode);
			else if (user.Identity.IsAdmin || user.HasRight(NSurveyRights.AccessLibrary))
				destURL = string.Format("{0}&surveyid={1}&menuindex={2}",LibraryDirectoryHyperLink, surveyId, (int)NavigationMenuItems.TheLibrary);
			else if (user.Identity.IsAdmin || user.HasRight(NSurveyRights.AccessSecuritySettings))
				destURL = string.Format("{0}&surveyid={1}&menuindex={2}",SurveySecurityLink, surveyId, (int)NavigationMenuItems.SurveySecurity);
			else if (user.Identity.IsAdmin || user.HasRight(NSurveyRights.AccessUserManager))
				destURL = string.Format("{0}&surveyid={1}&menuindex={2}",UsersManagerHyperLink, surveyId, (int)NavigationMenuItems.UsersManager);

			HttpContext.Current.Response.Redirect(destURL);
		}

	}
}
