/**************************************************************************************************

	NSurvey - The web survey and form engine
	Copyright (c) 2004, Thomas Zumbrunn. (http://www.nsurvey.org)

	Dotnetnuke PA module 
	Copyright (c) 2004, Oliver Hine (oliver.hine@gmail.com)

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


************************************************************************************************/
using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Votations.NSurvey.DataAccess;
using Votations.NSurvey.Data;
using Votations.NSurvey.UserProvider;
using Votations.NSurvey.WebAdmin.UserControls;
using DotNetNuke;

namespace Votations.NSurvey.WebAdmin
{
	public class UserManager: PageBase
	{
		protected System.Web.UI.WebControls.Label MessageLabel;
		protected HeaderControl Header;
		protected System.Web.UI.WebControls.Literal UsersManagerTitle;
		protected System.Web.UI.WebControls.Literal UserToEditLabel;
		protected System.Web.UI.WebControls.DropDownList UsersDropDownList;
		protected System.Web.UI.WebControls.HyperLink CreateUserHyperLink;
		protected UsersOptionsControl UsersOptions;

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		///		Required method for Designer support - do not modify
		///		the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.UsersDropDownList.SelectedIndexChanged += new System.EventHandler(this.User_IndexChanged);
			this.Load += new System.EventHandler(this.Page_Load);
		}
		#endregion

		private void Page_Load(object sender, System.EventArgs e)
		{
			try
			{
				SetupSecurity();
				LocalizePage();

				if (!Page.IsPostBack)
				{
					BindFields();
				}

				if (UsersDropDownList.SelectedValue != "0")
				{
					UsersOptions.UserId = int.Parse(UsersDropDownList.SelectedValue);
					UsersOptions.Visible = true;
				}

				UsersOptions.OptionChanged += new EventHandler(UsersManager_OptionChanged);
			}
			catch( Exception exc)
			{
				DotNetNuke.Services.Exceptions.Exceptions.ProcessModuleLoadException(this,exc);
			}
		}

		private void SetupSecurity()
		{
			CheckRight(NSurveyRights.AccessUserManager, true);
			IsSingleUserMode(true);
		}

		private void LocalizePage()
		{
			UsersManagerTitle.Text = GetPageResource("UsersManagerTitle");
			CreateUserHyperLink.Text = GetPageResource("CreateUserHyperLink");
			UserToEditLabel.Text = GetPageResource("UserToEditLabel");
		}

		/// <summary>
		/// Get the current DB data and fill 
		/// the fields with them
		/// </summary>
		private void BindFields()
		{
			Header.SurveyId = SurveyId;
			
			CreateUserHyperLink.Visible = _userProvider is INSurveyUserProvider;
			CreateUserHyperLink.NavigateUrl = UINavigator.UserCreatorHyperLink + "?surveyid="+SurveyId+"&menuindex=" + MenuIndex;

			UsersDropDownList.DataSource = _userProvider.GetAllUsersList();
			UsersDropDownList.DataMember = "Users";
			UsersDropDownList.DataTextField = "UserName";
			UsersDropDownList.DataValueField = "UserId";
			UsersDropDownList.DataBind();

			if (UsersDropDownList.Items.Count > 0)
			{
				UsersDropDownList.Items.Insert(0, 
					new ListItem(GetPageResource("SelectUserMessage"),"0"));
			}
			else
			{
				UsersDropDownList.Items.Insert(0, 
					new ListItem(GetPageResource("CreateAUserMessage"),"0"));
			}
		}


		/// <summary>
		///	Triggered by the type option user control
		///	in case anything was changed we will need to
		///	rebind to display the updates to the user 
		/// </summary>
		private void UsersManager_OptionChanged(object sender, EventArgs e)
		{
			BindFields();
		}

		private void User_IndexChanged(object sender, System.EventArgs e)
		{
			if (UsersDropDownList.SelectedValue != "0")
			{
				UsersOptions.UserId = int.Parse(UsersDropDownList.SelectedValue);
				UsersOptions.BindFields();
			}
			else
			{
				UsersOptions.UserId = -1;
				UsersOptions.Visible = false;
			}
		}

		private IUserProvider _userProvider = UserFactory.Create();

	}
}
